package net.lensfrex.disillusion.client.controller;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.jfoenix.controls.*;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Scene;
import javafx.scene.control.ToggleGroup;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.stage.Stage;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.ClientMain;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.page.employer.EmployerMainController;
import net.lensfrex.disillusion.client.controller.page.student.StudentMainController;
import net.lensfrex.disillusion.client.util.Consumer;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;
import okhttp3.OkHttpClient;
import okhttp3.Response;

import java.net.URL;

@Slf4j
public class LoginMainController {

    @FXML
    private StackPane rootPane;

    @FXML
    private JFXTextField usernameTextField;

    @FXML
    private JFXPasswordField passwordTextField;

    private ThreadSafeDialogMaker dialogMaker;

    private Stage mainStage;

    public void initialize() {
        dialogMaker = new ThreadSafeDialogMaker(rootPane);
    }

    public void setMainStage(Stage mainStage) {
        this.mainStage = mainStage;
    }

    @FXML
    void doLogin(ActionEvent event) {
        new OkhttpUtil().url(Constants.UserRequestUrls.LOGIN_URL)
                .form("username", usernameTextField.getText())
                .form("password", passwordTextField.getText())
                .failCallback(e -> Platform.runLater(() -> {
                    dialogMaker.createMessageDialog("啊哈", "登录时发生了网络问题：\n" + e);
                }))
                .successCallback(loginSuccessCallback)
                .executeAsync(OkhttpUtil.RequestMethod.POST);
    }

    private final Consumer<Response> loginSuccessCallback = response -> {
        ResponseHandler.Result result = ResponseHandler.parse(response, s -> dialogMaker.createMessageDialog("啊哈", nullSafe(s)));
        if (!result.success) {
            return;
        }

        JSONObject data = JSONUtil.parseObj(result.data);
        String token = data.getStr("token");
        String userType = data.getStr("user_type");

        switch (userType) {
            case "0" -> Platform.runLater(() -> this.jumpStudentMainWindow(usernameTextField.getText(), token));
            case "1" -> Platform.runLater(() -> this.jumpEmployMainWindow(usernameTextField.getText(), token));
        }
    };

    private static final URL studentMainWindowUrl = ClientMain.class.getResource("fxml/student/studentMain.fxml");

    private static final URL employerMainWindowUrl = ClientMain.class.getResource("fxml/employer/employerMain.fxml");

    private void jumpStudentMainWindow(String username, String token) {
        try {
            FXMLLoader fxmlLoader = new FXMLLoader(studentMainWindowUrl);
            Scene scene = new Scene(fxmlLoader.load(), 1572, 832);
            mainStage.setScene(scene);
            scene.getStylesheets().add(ClientMain.class.getResource("css/style.css").toExternalForm());

            StudentMainController.InitData initData = StudentMainController.InitData.builder()
                    .username(username)
                    .token(token)
                    .build();

            StudentMainController controller = fxmlLoader.getController();
            controller.setData(initData);

            mainStage.setTitle("disillusion - student");
            mainStage.setOnCloseRequest(v -> System.exit(0));
            mainStage.setX(100);
            mainStage.setY(100);
            mainStage.show();
        } catch (Exception e) {
            e.printStackTrace();
            dialogMaker.createMessageDialog("Oppos...😱＞︿＜", "程序内部出错...请联系@lensfrex这家伙\n调试信息：\n" + e);
        }
    }

    private void jumpEmployMainWindow(String username, String token) {
        try {
            FXMLLoader fxmlLoader = new FXMLLoader(employerMainWindowUrl);
            Scene scene = new Scene(fxmlLoader.load(), 1572, 832);
            mainStage.setScene(scene);
            scene.getStylesheets().add(ClientMain.class.getResource("css/style.css").toExternalForm());

            EmployerMainController controller = fxmlLoader.getController();
            controller.setData(token);

            mainStage.setTitle("disillusion - employer");
            mainStage.setOnCloseRequest(v -> System.exit(0));
            mainStage.setX(100);
            mainStage.setY(100);
            mainStage.show();
        } catch (Exception e) {
            e.printStackTrace();
            dialogMaker.createMessageDialog("Oppos...😱＞︿＜", "程序内部出错...请联系@lensfrex这家伙\n调试信息：\n" + e);
        }
    }

    @FXML
    void doRegister(ActionEvent event) {
        JFXRadioButton studentChoice = new JFXRadioButton("学生");
        studentChoice.setUserData(Constants.USER_TYPE_STUDENT);
        JFXRadioButton employerChoice = new JFXRadioButton("用人单位");
        employerChoice.setUserData(Constants.USER_TYPE_EMPLOYER);

        ToggleGroup userTypeGroup = new ToggleGroup();
        studentChoice.setToggleGroup(userTypeGroup);
        employerChoice.setToggleGroup(userTypeGroup);
        userTypeGroup.selectToggle(studentChoice);

        VBox choiceBox = new VBox(studentChoice, employerChoice);
        choiceBox.setSpacing(10);

        JFXButton cancelButton = new JFXButton("取消");
        JFXButton okButton = new JFXButton("好的");
        okButton.setOnAction(actionEvent -> {
            String userTypeString = userTypeGroup.getSelectedToggle().getUserData().toString();
            this.register(userTypeString);
        });

        dialogMaker.createDialog("选择要注册的用户类型", choiceBox, cancelButton, okButton);
    }

    private void register(String userTypeString) {
        new OkhttpUtil()
                .url(Constants.UserRequestUrls.REGISTER_URL)
                .form("username", usernameTextField.getText())
                .form("password", passwordTextField.getText())
                .form("userType", userTypeString)
                .failCallback(e -> {
                    dialogMaker.createMessageDialog("啊哈", "注册时发生了网络问题：\n" + e);
                })
                .successCallback(response -> {
                    ResponseHandler.Result result = ResponseHandler.parse(response, msg -> dialogMaker.createMessageDialog("啊哈", nullSafe(msg)));
                    if (!result.success) {
                        return;
                    }

                    dialogMaker.createMessageDialog("Message", "好了");
                })
                .executeAsync(OkhttpUtil.RequestMethod.POST);
    }

    private static String nullSafe(String str) {
        return str == null ? "null" : str;
    }
}
