package net.lensfrex.disillusion.client.controller.components;

import com.jfoenix.svg.SVGGlyph;
import javafx.animation.KeyFrame;
import javafx.animation.KeyValue;
import javafx.animation.Timeline;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.control.Button;
import javafx.scene.paint.Color;
import javafx.util.Duration;
import net.lensfrex.disillusion.client.ClientMain;
import net.lensfrex.disillusion.client.controller.components.employer.offer.EmployerOfferDetailPane;

import java.io.IOException;

import static javafx.animation.Interpolator.EASE_BOTH;

public class ComponentFactory {
    public static <T extends Node> T loadComponent(String location, Class<T> clazz)
            throws IOException, InstantiationException, IllegalAccessException {

        T controller = clazz.newInstance();
        return loadComponent(location, controller);
    }

    public static <T extends Node> T loadComponent(String location, T controller)
            throws IOException {

        FXMLLoader loader = new FXMLLoader(ClientMain.class.getResource(location));
        return loadComponent(loader, controller);
    }

    public static <T extends Node> T loadComponent(FXMLLoader loader, T controller)
            throws IOException {

        loader.setRoot(controller);
        loader.setController(controller);

        return loader.load();
    }

    public static void setCardIcon(Button button, String svg) {
        SVGGlyph glyph = new SVGGlyph(-1, "send", svg, Color.WHITE);
        glyph.setSize(20, 20);
        button.setGraphic(glyph);

        Timeline animation = new Timeline(new KeyFrame(Duration.millis(240),
                new KeyValue(button.scaleXProperty(), 1, EASE_BOTH),
                new KeyValue(button.scaleYProperty(), 1, EASE_BOTH)));
        animation.setDelay(Duration.millis(100 + 100));
        animation.play();
    }
}
