package net.lensfrex.disillusion.client.controller.components.employer.application;

import cn.hutool.json.JSONUtil;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.text.Text;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.data.response.ApplicationResponse;
import net.lensfrex.disillusion.client.data.response.PublicStudentInfoResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.IOException;

@Slf4j
public class ApplicationListItem extends HBox {
    private final ApplicationResponse application;
    private final ThreadSafeDialogMaker dialogMaker;
    private final String token;

    private PublicStudentInfoResponse studentInfo;

    @FXML
    private Text studentNameText;

    @FXML
    private Text describeText;

    public ApplicationListItem(ApplicationResponse application,
                               ThreadSafeDialogMaker dialogMaker, String token) {

        this.application = application;
        this.dialogMaker = dialogMaker;
        this.token = token;
    }

    public void initialize() {
        this.getStudentInfo();
    }

    private void getStudentInfo() {
        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_PUBLIC_INFO_VIEW_URL)
                .addQueryParam("user", application.getPublisherId())
                .failCallback(e -> log.warn("获取学生信息时发生网络错误：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> log.warn("获取学生信息不成功：\n" + s))
                        .successCallback(s -> {
                            studentInfo = JSONUtil.toBean(s, PublicStudentInfoResponse.class);
                            studentNameText.setText(studentInfo.getRealName());
                            describeText.setText(studentInfo.getShortDescribe());
                        })
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    @FXML
    void dealApplication(ActionEvent event) throws IOException, InstantiationException, IllegalAccessException {
        ApplicationDealPane dealPane = ComponentFactory.loadComponent("fxml/employer/application/applicationDealPane.fxml",
                ApplicationDealPane.class);
        dialogMaker.createDialogWithOKAndCancel("处理Offer申请", dealPane, event1 -> this.dealApplication(dealPane.getOpinion()));
    }

    private void dealApplication(String opinion) {
        new OkhttpUtil()
                .url(Constants.ApplicationRequestUrls.APPLICATION_DEAL_URL)
                .token(token)
                .addQueryParam("id", application.getId())
                .addQueryParam("opinion", opinion)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "处理offer申请时出现网络错误：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "处理offer申请失败：\n" + s))
                        .successCallback(s -> dialogMaker.createMessageDialog("好好好", "处理offer申请成功：\n" + opinion))
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.POST);
    }

    @FXML
    void viewResume(ActionEvent event) throws IOException {
        ResumeDownloadPane downloadPane = ComponentFactory.loadComponent("fxml/employer/application/resumeDownloadPane.fxml",
                new ResumeDownloadPane(application.getResumeId(), studentInfo.getRealName(), dialogMaker, token));
        dialogMaker.createDialogWithOneBtn("下载简历", downloadPane);
    }

    @FXML
    void viewStudentInfo(MouseEvent event) throws IOException {
        StudentInfoPage page = ComponentFactory.loadComponent("fxml/employer/application/studentProfile.fxml",
                new StudentInfoPage(studentInfo));
        dialogMaker.createDialogWithOneBtn("学生信息", page);
    }
}
