package net.lensfrex.disillusion.client.controller.components.employer.application;

import cn.hutool.json.JSONUtil;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.layout.StackPane;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.data.response.ApplicationResponse;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.IOException;
import java.util.List;

@Slf4j
public class ApplicationListPage extends StackPane {
    @FXML
    private VBox applicationContainer;

    private final OfferResponse offer;

    private final ThreadSafeDialogMaker dialogMaker;
    private final String token;

    public ApplicationListPage(ThreadSafeDialogMaker dialogMaker, String token, OfferResponse offer) {
        this.dialogMaker = dialogMaker;
        this.token = token;
        this.offer = offer;
    }

    public void initialize() {
        new OkhttpUtil()
                .url(Constants.ApplicationRequestUrls.APPLICATION_EMPLOYER_VIEW_URL)
                .token(token)
                .addQueryParam("offer_id", offer.getId())
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "获取求职申请列表时发生了网络问题：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "获取求职申请不成功：\n" + s))
                        .successCallback(s -> Platform.runLater(() -> this.showApplicationList(s)))
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void showApplicationList(String jsonData) {
        try {
            List<ApplicationResponse> applications = JSONUtil.toList(jsonData, ApplicationResponse.class);
            if (applications.isEmpty()) {
                Text text = new Text("这里空空如也...");
                text.setFont(Font.font(20));
                applicationContainer.getChildren().add(text);
            }

            ObservableList<ApplicationListItem> items = FXCollections.observableArrayList();
            for (ApplicationResponse application : applications) {
                ApplicationListItem item = ComponentFactory.loadComponent("fxml/employer/application/applicationListItem.fxml",
                        new ApplicationListItem(application, dialogMaker, token));
                items.add(item);
            }

            applicationContainer.getChildren().addAll(items);
        } catch (IOException e) {
            e.printStackTrace();
            log.error("加载offer申请页面时出错");
        }
    }
}