package net.lensfrex.disillusion.client.controller.components.employer.application;

import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.StackPane;
import javafx.scene.text.Text;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

@Slf4j
public class ApplicationOfferCard extends StackPane {
    private final OfferResponse offer;

    private ThreadSafeDialogMaker dialogMaker;

    private String token;

    private static final String[] colors = new String[]{
            "-fx-background-color: #8F3F7E",
            "-fx-background-color: #B5305F",
            "-fx-background-color: #CE584A",
            "-fx-background-color: #DB8D5C",
            "-fx-background-color: #DA854E",
            "-fx-background-color: #E9AB44",
            "-fx-background-color: #99C286",
            "-fx-background-color: #01A05E",
            "-fx-background-color: #4A8895",
            "-fx-background-color: #16669B",
            "-fx-background-color: #2F65A5",
    };

    public ApplicationOfferCard(OfferResponse offer) {
        this.setUserData(offer);
        this.offer = offer;
    }

    public void loadData(String token, ThreadSafeDialogMaker dialogMaker) {
        this.token = token;
        this.dialogMaker = dialogMaker;

        // 查询offer招募进度
        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_PROGRESS_URL)
                .token(token)
                .addQueryParam("id", offer.getId())
                .failCallback(e -> log.warn("获取offer进度时出现网络错误：", e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> log.warn("获取offer进度不成功：" + s))
                        .successCallback(s -> processText.setText("招募进度：" + s))
                        .process()
                )
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    public void initialize() {
        String color = colors[Math.abs(offer.getJobName().hashCode()) % colors.length];
        header.setStyle(header.getStyle() + color);

        jobName.setText(offer.getJobName());
    }

    @FXML
    private StackPane rootPane;

    @FXML
    private StackPane header;

    @FXML
    private Label jobName;

    @FXML
    private StackPane body;

    @FXML
    private Text processText;

    @FXML
    void viewApplications(MouseEvent event) {
        new OkhttpUtil()
                .url(Constants.ApplicationRequestUrls.APPLICATION_EMPLOYER_VIEW_URL)
                .token(token)
                .addQueryParam("offer_id", offer.getId())
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "获取求职申请列表时发生了网络问题：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "获取求职申请不成功：\n" + s))
                        .successCallback(s -> showApplicationListPage())
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void showApplicationListPage() {
        try {
            ApplicationListPage applicationListPage = ComponentFactory.loadComponent("fxml/employer/application/applicationListPage.fxml",
                    new ApplicationListPage(dialogMaker, token, offer));
            dialogMaker.createDialogWithOneBtn(String.format("投递给 '%s' 的求职申请", offer.getJobName()), applicationListPage);
        } catch (Exception e) {
            log.error("显示简历申请列表页面时出错", e);
        }
    }
}