package net.lensfrex.disillusion.client.controller.components.employer.application;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.io.FileUtil;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.layout.VBox;
import javafx.stage.DirectoryChooser;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.File;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;

public class ResumeDownloadPane extends VBox {
    @FXML
    private VBox rootPane;

    private final String fileId;
    private final String fileName;

    private final ThreadSafeDialogMaker dialogMaker;
    private final String token;

    public ResumeDownloadPane(String fileId, String fileName, ThreadSafeDialogMaker dialogMaker, String token) {
        this.fileId = fileId;
        this.fileName = fileName;
        this.dialogMaker = dialogMaker;
        this.token = token;
    }

    @FXML
    void download(ActionEvent event) {
        DirectoryChooser directoryChooser = new DirectoryChooser();
        directoryChooser.setTitle("要将简历文件保存到哪？");
        File saveFile = directoryChooser.showDialog(rootPane.getScene().getWindow());
        if (saveFile == null) {
            return;
        }

        downloadResume(saveFile.getAbsolutePath(), fileName);
    }

    private void downloadResume(String saveDir, String fileName) {
        Path target = Paths.get(saveDir, fileName);

        new OkhttpUtil()
                .url(Constants.ResumeRequestUrls.RESUME_FILE_URL)
                .addHeader("token", token)
                .addQueryParam("id", fileId)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "下载简历文件时出现网络问题：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "下载简历文件不成功：" + s))
                        .successCallback(bytes -> {
                            Files.write(target, Base64.decode(bytes), StandardOpenOption.CREATE, StandardOpenOption.WRITE);
                            Path saved = FileUtil.rename(target,
                                    String.format("%s_%s.%s", fileName, fileId, FileUtil.getType(target.toFile())),
                                    true);
                            dialogMaker.createMessageDialog("好好好", "下载完成，保存至：\n" + saved);
                        })
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }
}
