package net.lensfrex.disillusion.client.controller.components.employer.application;

import com.jfoenix.controls.JFXTextArea;
import com.jfoenix.controls.JFXTextField;
import javafx.application.Platform;
import javafx.fxml.FXML;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.layout.HBox;
import javafx.scene.text.Text;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.data.response.PublicStudentInfoResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import okhttp3.Response;

import java.io.ByteArrayInputStream;

@Slf4j
public class StudentInfoPage extends HBox {
    @FXML
    private HBox rootPane;

    @FXML
    private ImageView avatarImageView;

    @FXML
    private Text usernameText;

    @FXML
    private Text signText;

    @FXML
    private JFXTextField realNameTextField;

    @FXML
    private JFXTextField ageTextField;

    @FXML
    private Text sexText;

    @FXML
    private JFXTextField contactTextField;

    @FXML
    private JFXTextField signTextField;

    @FXML
    private JFXTextArea describeTextArea;

    private final PublicStudentInfoResponse studentInfo;

    public StudentInfoPage(PublicStudentInfoResponse studentInfo) {
        this.studentInfo = studentInfo;
    }

    @FXML
    public void initialize() {
        realNameTextField.setText(studentInfo.getRealName());
        signText.setText(studentInfo.getShortDescribe());
        signTextField.setText(studentInfo.getShortDescribe());
        describeTextArea.setText(studentInfo.getDescribe());
        contactTextField.setText(studentInfo.getPhoneNumber());
        ageTextField.setText(String.valueOf(studentInfo.getAge()));
        sexText.setText(studentInfo.getSex() == 0 ? "男" : "女");

        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_AVATAR_VIEW_URL)
                .addQueryParam("user", studentInfo.getId())
                .failCallback(e -> log.warn("获取头像时发生网络错误：\n" + e))
                .successCallback(response -> Platform.runLater(() -> setAvatar(avatarImageView, response)))
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    public static void setAvatar(ImageView avatarImageView, Response response) {
        try {
            assert response.body() != null;
            byte[] data = response.body().bytes();

            setAvatar(avatarImageView, data);
        } catch (Exception e) {
            log.warn("获取头像数据时出现异常", e);
        }
    }

    public static void setAvatar(ImageView avatarImageView, byte[] data) {
        Image avatarImage = new Image(new ByteArrayInputStream(data));
        avatarImageView.setImage(avatarImage);
    }
}
