package net.lensfrex.disillusion.client.controller.components.employer.offer;

import com.jfoenix.controls.JFXButton;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.StackPane;
import javafx.scene.text.Text;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.data.request.OfferPublishRequest;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.util.Consumer;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.IOException;

@Slf4j
public class EmployerOfferCard extends StackPane {
    private final OfferResponse offer;

    private ThreadSafeDialogMaker dialogMaker;

    private String token;

    private Consumer<String> deleteCallback;
    private Consumer<String> updateCallback;

    private static final String[] colors = new String[]{
            "-fx-background-color: #8F3F7E",
            "-fx-background-color: #B5305F",
            "-fx-background-color: #CE584A",
            "-fx-background-color: #DB8D5C",
            "-fx-background-color: #DA854E",
            "-fx-background-color: #E9AB44",
            "-fx-background-color: #99C286",
            "-fx-background-color: #01A05E",
            "-fx-background-color: #4A8895",
            "-fx-background-color: #16669B",
            "-fx-background-color: #2F65A5",
    };

    public EmployerOfferCard(OfferResponse offer) {
        this.setUserData(offer);
        this.offer = offer;
    }

    public void loadData(String token, ThreadSafeDialogMaker dialogMaker,
                         Consumer<String> deleteCallback, Consumer<String> updateCallback) {
        this.token = token;
        this.dialogMaker = dialogMaker;
        this.deleteCallback = deleteCallback;
        this.updateCallback = updateCallback;

        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_PROGRESS_URL)
                .token(token)
                .addQueryParam("id", offer.getId())
                .failCallback(e -> log.warn("获取offer进度时出现网络错误：", e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> log.warn("获取offer进度不成功：" + s))
                        .successCallback(s -> processText.setText("招募进度：" + s))
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    public void initialize() {
        String color = colors[Math.abs(offer.getJobName().hashCode()) % colors.length];
        header.setStyle(header.getStyle() + color);

        jobName.setText(offer.getJobName());

        ComponentFactory.setCardIcon(offerDeleteBtn, Constants.Svg.deleteSvg);
    }

    @FXML
    private StackPane rootPane;

    @FXML
    private StackPane header;

    @FXML
    private Label jobName;

    @FXML
    private StackPane body;

    @FXML
    private Text processText;

    @FXML
    private JFXButton offerDeleteBtn;

    @FXML
    void deleteOffer(ActionEvent event) {
        dialogMaker.createDialogWithOKAndCancel("Really？", String.format("真的要删除Offer '%s'", offer.getJobName()), event1 -> new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_DELETE_URL)
                .token(token)
                .addQueryParam("id", offer.getId())
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "删除offer时出现网络问题：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "删除offer不成功：\n" + s))
                        .successCallback(deleteCallback)
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.POST)
        );
    }

    @FXML
    void viewDetail(MouseEvent event) throws IOException {
        EmployerOfferDetailPane offerDetailPane =
                ComponentFactory.loadComponent("fxml/employer/offer/offerDetailPane.fxml", new EmployerOfferDetailPane(offer));

        dialogMaker.createDialogWithOKAndCancel("更新Offer信息", offerDetailPane, e -> this.updateOffer(offerDetailPane.getData()));
    }

    private void updateOffer(OfferPublishRequest offerPublishRequest) {
        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_UPDATE_URL)
                .token(token)
                .addQueryParam("id", offer.getId())
                .body(offerPublishRequest)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "更新offer信息时发生了网络问题：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "更新offer信息不成功：\n" + s))
                        .successCallback(s -> updateCallback.accept(s))
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.POST);
    }
}