package net.lensfrex.disillusion.client.controller.components.student.application;

import cn.hutool.json.JSONUtil;
import com.jfoenix.controls.JFXButton;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Color;
import javafx.scene.text.Text;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.controller.components.student.offer.OfferDetailPane;
import net.lensfrex.disillusion.client.data.response.ApplicationResponse;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.util.Consumer;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.IOException;

@Slf4j
public class ApplicationCard extends StackPane {
    private static final String[] colors = new String[]{
            "-fx-background-color: #8F3F7E",
            "-fx-background-color: #B5305F",
            "-fx-background-color: #CE584A",
            "-fx-background-color: #DB8D5C",
            "-fx-background-color: #DA854E",
            "-fx-background-color: #E9AB44",
            "-fx-background-color: #99C286",
            "-fx-background-color: #01A05E",
            "-fx-background-color: #4A8895",
            "-fx-background-color: #16669B",
            "-fx-background-color: #2F65A5",
    };

    private final String token;
    private final ThreadSafeDialogMaker dialogMaker;
    private final ApplicationResponse application;

    private final Consumer<String> deleteCallback;

    private OfferResponse offer;

    public ApplicationCard(String token,
                           ThreadSafeDialogMaker dialogMaker,
                           ApplicationResponse application,
                           Consumer<String> deleteCallback) {

        this.token = token;
        this.dialogMaker = dialogMaker;
        this.application = application;
        this.deleteCallback = deleteCallback;
    }

    @FXML
    private StackPane rootPane;

    @FXML
    private StackPane header;

    @FXML
    private Label jobName;

    @FXML
    private StackPane body;

    @FXML
    private Text statusText;

    @FXML
    private JFXButton applicationDeleteBtn;

    public void initialize() {
        String color = colors[Math.abs(application.getOfferId().hashCode()) % colors.length];
        header.setStyle(header.getStyle() + color);

        this.showJobName();

        statusText.setText("状态：" + getStatusText(application.getStatus()));
        if ("REJECTED".equals(application.getStatus())){
            statusText.setFill(Color.valueOf("#C62828"));
        } else if ("ACCEPTED".equals(application.getStatus())) {
            statusText.setFill(Color.valueOf("#2E7D32"));
        }
        ComponentFactory.setCardIcon(applicationDeleteBtn, Constants.Svg.deleteSvg);
    }

    private void showJobName() {
        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.GET_OFFER_URL)
                .addQueryParam("id", application.getOfferId())
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "获取职位申请时发生了错误：" + e))
                .successCallback(response -> new ResponseHandler(response).successCallback(data -> {
                    this.offer = JSONUtil.toBean(data, OfferResponse.class);
                    Platform.runLater(() -> jobName.setText(offer.getJobName()));
                }).process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private String getStatusText(String status) {
        switch (status) {
            case "WAITING":
                return "等待中";
            case "ACCEPTED":
                return "已接受";
            case "REJECTED":
                return "已拒绝";
            default:
                return status;
        }
    }

    @FXML
    void deleteApplication(ActionEvent event) {
        dialogMaker.createDialogWithOKAndCancel("确认操作？", String.format("真的要删除 '%s' 的申请吗？", offer.getJobName()),
                e -> new OkhttpUtil()
                        .url(Constants.ApplicationRequestUrls.APPLICATION_DELETE_URL)
                        .addQueryParam("id", application.getId())
                        .token(token)
                        .failCallback(ex -> dialogMaker.createMessageDialog("啊哈", "获取职位申请时发生了错误：" + ex))
                        .successCallback(response -> new ResponseHandler(response)
                                .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "删除offer申请不成功：\n" + s))
                                .successCallback(deleteCallback)
                                .process())
                        .executeAsync(OkhttpUtil.RequestMethod.POST));
    }

    @FXML
    void viewDetail(MouseEvent event) throws IOException {
        OfferDetailPane detailPage =
                ComponentFactory.loadComponent("fxml/student/offer/offerDetail.fxml", new OfferDetailPane(dialogMaker, this.offer));
        dialogMaker.createDialogWithOneBtn("Offer详情", detailPage);
    }
}
