package net.lensfrex.disillusion.client.controller.components.student.application;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.jfoenix.controls.JFXComboBox;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.layout.HBox;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.data.ResponseCode;
import net.lensfrex.disillusion.client.data.response.ResumeResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
public class ApplicationPostPane extends HBox {
    private final ObservableList<String> resumeChoiceList = FXCollections.observableArrayList();
    private Map<String, String> resumeNameIdMap;

    @FXML
    private JFXComboBox<String> resumeComboBox;

    private final String token;

    public ApplicationPostPane(String token) {
        this.token = token;
    }

    public void initialize() {
        this.getResumes();
    }

    public String getChooseResumeId() {
        String selectedName = resumeComboBox.getSelectionModel().getSelectedItem();
        if (selectedName == null) {
            return null;
        }

        return resumeNameIdMap.get(selectedName);
    }

    private void getResumes() {
        new OkhttpUtil()
                .url(Constants.ResumeRequestUrls.RESUME_GET_ALL_URL)
                .token(token)
                .successCallback(response -> {
                    assert response.body() != null;
                    String responseString = response.body().string();
                    JSONObject responseJson = JSONUtil.parseObj(responseString);

                    Integer code = responseJson.getInt("code");
                    if (code == null || !code.equals(ResponseCode.SUCCESS.getCode())) {
                        String message = responseJson.getStr("message");
                        log.warn("获取简历时发生错误：" + message);
                    }

                    List<ResumeResponse> resumes = JSONUtil.toList(responseJson.getJSONArray("data"), ResumeResponse.class);
                    Platform.runLater(() -> this.addResumeItems(resumes));
                })
                .failCallback(e -> log.warn("获取简历时发生错误：" + e))
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void addResumeItems(List<ResumeResponse> resumes) {
        resumeNameIdMap = new HashMap<>(resumes.size());
        for (ResumeResponse resume : resumes) {
            if (resume == null) {
                continue;
            }
            resumeChoiceList.add(resume.getTitle());
            resumeNameIdMap.put(resume.getTitle(), resume.getId());
        }

        resumeComboBox.setItems(resumeChoiceList);
        resumeComboBox.getSelectionModel().select(0);
    }
}
