package net.lensfrex.disillusion.client.controller.components.student.offer;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.io.FileUtil;
import com.jfoenix.controls.JFXTextArea;
import com.jfoenix.controls.JFXTextField;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.text.Text;
import javafx.stage.FileChooser;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.data.response.EmployerInfoResponse;
import net.lensfrex.disillusion.client.data.response.PublicEmployerInfoResponse;
import net.lensfrex.disillusion.client.util.ImageUtil;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;
import okhttp3.Response;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.function.Consumer;

@Slf4j
public class EmployerInfoPage extends HBox {
    @FXML
    private HBox rootPane;

    @FXML
    private ImageView avatarImageView;

    @FXML
    private Text usernameText;

    @FXML
    private Text shortDescribeText;

    @FXML
    private JFXTextField employerNameTextField;

    @FXML
    private JFXTextField industryTextField;

    @FXML
    private JFXTextField contactTextField;

    @FXML
    private JFXTextField addressTextField;

    @FXML
    private JFXTextField shortDescribeTextField;

    @FXML
    private JFXTextArea describeTextArea;

    private File newAvatarFile;

    private ThreadSafeDialogMaker dialogMaker;

    private final PublicEmployerInfoResponse employerInfo;

    public EmployerInfoPage(PublicEmployerInfoResponse employerInfo) {
        this.employerInfo = employerInfo;
    }

    public void initialize() {
        this.updateDisplay();

        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_AVATAR_VIEW_URL)
                .addQueryParam("user", employerInfo.getId())
                .successCallback(response -> Platform.runLater(() -> setAvatar(avatarImageView, response)))
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void updateDisplay() {
        String name = employerInfo.getName();
        String place = employerInfo.getPlace();
        String contract = employerInfo.getContract();
        String shortDescribe = employerInfo.getShortDescribe();
        String describe = employerInfo.getDescribe();
        String industry = employerInfo.getIndustry();

        usernameText.setText(name);
        shortDescribeText.setText(shortDescribe);

        employerNameTextField.setText(name);
        industryTextField.setText(industry);
        contactTextField.setText(contract);
        addressTextField.setText(place);
        shortDescribeTextField.setText(shortDescribe);
        describeTextArea.setText(describe);
    }

    public static void setAvatar(ImageView avatarImageView, Response response) {
        try {
            assert response.body() != null;
            byte[] data = response.body().bytes();

            setAvatar(avatarImageView, data);
        } catch (Exception e) {
            log.warn("获取头像数据时出现异常", e);
        }
    }

    public static void setAvatar(ImageView avatarImageView, byte[] data) {
        Image avatarImage = new Image(new ByteArrayInputStream(data));
        avatarImageView.setImage(avatarImage);
    }
}
