package net.lensfrex.disillusion.client.controller.components.student.offer;

import cn.hutool.json.JSONUtil;
import com.jfoenix.controls.JFXTextArea;
import javafx.fxml.FXML;
import javafx.scene.layout.VBox;
import javafx.scene.text.Text;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.data.response.PublicEmployerInfoResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.IOException;

@Slf4j
public class OfferDetailPane extends VBox {

    private final ThreadSafeDialogMaker dialogMaker;

    public OfferDetailPane(ThreadSafeDialogMaker dialogMaker, OfferResponse offer,
                           PublicEmployerInfoResponse employerInfo) {
        this.dialogMaker = dialogMaker;
        this.offer = offer;
        this.employerInfo = employerInfo;
    }

    public OfferDetailPane(ThreadSafeDialogMaker dialogMaker, OfferResponse offer) {
        this.dialogMaker = dialogMaker;
        this.offer = offer;
        this.getEmployerInfo();
    }

    private void getEmployerInfo() {
        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_PUBLIC_INFO_VIEW_URL)
                .addQueryParam("user", offer.getPublisherId())
                .failCallback(e -> log.warn("获取企业信息时发生网络错误：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> log.warn("获取企业信息不成功：\n" + s))
                        .successCallback(s -> {
                            employerInfo = JSONUtil.toBean(s, PublicEmployerInfoResponse.class);
                            employerText.setText(employerInfo.getName());
                        })
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    public void initialize() {
        if (employerInfo != null) {
            employerText.setText(employerInfo.getName());
        }

        jobNameText.setText(offer.getJobName());
        placeText.setText(offer.getPlace());
        requirementText.setText(offer.getRequirement());
        salaryText.setText(offer.getSalary().toString());
        targetNumberText.setText(offer.getTargetNumber().toString());
        detailText.setText(offer.getDetail());
    }

    @FXML
    void showEmployerDetail() throws IOException {
        EmployerInfoPage employerInfoPage =
                ComponentFactory.loadComponent("fxml/student/offer/employerProfile.fxml", new EmployerInfoPage(employerInfo));
        dialogMaker.createDialogWithOneBtn("企业详情", employerInfoPage);
    }

    private final OfferResponse offer;
    private PublicEmployerInfoResponse employerInfo;

    @FXML
    private Text jobNameText;

    @FXML
    private Text salaryText;

    @FXML
    private Text employerText;

    @FXML
    private Text placeText;

    @FXML
    private Text targetNumberText;

    @FXML
    private Text requirementText;

    @FXML
    private JFXTextArea detailText;
}
