package net.lensfrex.disillusion.client.controller.components.student.resume;

import cn.hutool.core.codec.Base64;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.jfoenix.controls.JFXButton;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.Label;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.StackPane;
import javafx.scene.text.Text;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.data.ResponseCode;
import net.lensfrex.disillusion.client.data.request.ResumeRequest;
import net.lensfrex.disillusion.client.data.response.ResumeResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;
import okhttp3.*;
import org.jetbrains.annotations.NotNull;

import java.io.File;
import java.io.IOException;
import java.util.function.Consumer;

public class ResumeCard extends StackPane {
    private static final String[] colors = new String[]{
            "-fx-background-color: #8F3F7E",
            "-fx-background-color: #B5305F",
            "-fx-background-color: #CE584A",
            "-fx-background-color: #DB8D5C",
            "-fx-background-color: #DA854E",
            "-fx-background-color: #E9AB44",
            "-fx-background-color: #99C286",
            "-fx-background-color: #01A05E",
            "-fx-background-color: #4A8895",
            "-fx-background-color: #16669B",
            "-fx-background-color: #2F65A5",
    };

    private static final OkHttpClient okhttpClient = Constants.globalClient;

    private final ResumeResponse resume;

    private final String token;

    private final ThreadSafeDialogMaker dialogMaker;

    private final Consumer<Response> updateCallback;
    private final Consumer<Response> deleteCallback;

    public ResumeCard(ResumeResponse resume, String token, ThreadSafeDialogMaker dialogMaker,
                      Consumer<Response> updateCallback, Consumer<Response> deleteCallback) {
        this.resume = resume;
        this.token = token;
        this.dialogMaker = dialogMaker;

        this.updateCallback = updateCallback;
        this.deleteCallback = deleteCallback;
    }

    @FXML
    private StackPane rootPane;

    @FXML
    private StackPane header;

    @FXML
    private Label resumeNameText;

    @FXML
    private StackPane body;

    @FXML
    private Text remarkText;

    @FXML
    private JFXButton resumeDeleteBtn;

    public void initialize() {
        String color = colors[Math.abs(resume.getTitle().hashCode()) % colors.length];
        header.setStyle(header.getStyle() + color);

        resumeNameText.setText(resume.getTitle());
        remarkText.setText(resume.getRemark());

        ComponentFactory.setCardIcon(resumeDeleteBtn, Constants.Svg.deleteSvg);
    }

    @FXML
    void showResumeDetail(MouseEvent event) throws IOException {
        ResumeUploadPane resumeUploadPane =
                ComponentFactory.loadComponent("fxml/student/resume/resumeUpload.fxml", new ResumeUploadPane(dialogMaker));

        ResumeUploadPane.InitData initData = new ResumeUploadPane.InitData();
        initData.setName(resume.getTitle());
        initData.setRemark(resume.getRemark());
        resumeUploadPane.loadData(initData);

        dialogMaker.createDialogWithOKAndCancel("更新简历", resumeUploadPane, e -> {
            try {
                ResumeRequest request = new ResumeRequest();
                request.setTitle(resumeUploadPane.getResumeName());
                request.setRemark(resumeUploadPane.getRemark());

                File resumeFile = resumeUploadPane.getResumeFile();
                if (resumeFile != null) {
                    request.setResumeFileBase64(Base64.encode(resumeFile));
                }

                this.updateResume(request);
            } catch (Exception ex) {
                ex.printStackTrace();
                dialogMaker.createMessageDialog("啊哈", "读取简历文件时发生错误...\n" + ex);
            }
        });
    }

    private void updateResume(ResumeRequest resumeRequest) {
        String json = JSONUtil.toJsonStr(resumeRequest);
        RequestBody requestBody = RequestBody.create(json, MediaType.get("application/json"));

        Request uploadRequest = new Request.Builder()
                .url(Constants.ResumeRequestUrls.RESUME_UPDATE_URL + "?id=" + resume.getId())
                .addHeader("token", token)
                .post(requestBody)
                .build();

        okhttpClient.newCall(uploadRequest).enqueue(resumeUpdateCallback);
    }

    private final Callback resumeUpdateCallback = new Callback() {
        @Override
        public void onFailure(@NotNull Call call, @NotNull IOException e) {
            dialogMaker.createMessageDialog("啊哈", "上传简历时发生了网络错误\n" + e);
        }

        @Override
        public void onResponse(@NotNull Call call, @NotNull Response response) throws IOException {
            assert response.body() != null;
            String responseString = response.body().string();
            JSONObject responseJson = JSONUtil.parseObj(responseString);

            Integer code = responseJson.getInt("code");
            if (code == null || !code.equals(ResponseCode.SUCCESS.getCode())) {
                String message = responseJson.getStr("message");
                dialogMaker.createMessageDialog("啊哈", "获取用户信息时出错了...\n" + message);
            } else {
                dialogMaker.createMessageDialog("好耶", "上传成功");
                Platform.runLater(() -> updateCallback.accept(response));
            }
        }
    };

    @FXML
    void deleteResume(ActionEvent event) {
        dialogMaker.createDialogWithOKAndCancel("Really？", String.format("真的要删除简历 '%s' 吗？", resume.getTitle()),
                e -> new OkhttpUtil()
                        .url(Constants.ResumeRequestUrls.RESUME_DELETE_URL)
                        .token(token)
                        .addQueryParam("id", resume.getId())
                        .failCallback(e1 -> dialogMaker.createMessageDialog("啊哈", "删除简历时发生了网络错误\n" + e1))
                        .successCallback(response -> new ResponseHandler(response)
                                .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "删除简历时出错了...\n" + s))
                                .successCallback(response1 -> Platform.runLater(() -> deleteCallback.accept(response)))
                                .process())
                        .executeAsync(OkhttpUtil.RequestMethod.POST));
    }
}
