package net.lensfrex.disillusion.client.controller.components.student.resume;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.RandomUtil;
import com.jfoenix.controls.JFXTextField;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.layout.VBox;
import javafx.scene.text.Text;
import javafx.stage.FileChooser;
import lombok.Data;
import lombok.EqualsAndHashCode;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.CommunicativeController;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.File;

public class ResumeUploadPane extends VBox implements CommunicativeController<ResumeUploadPane.InitData> {
    private final ThreadSafeDialogMaker dialogMaker;

    public ResumeUploadPane(ThreadSafeDialogMaker dialogMaker) {
        this.dialogMaker = dialogMaker;
    }

    @Override
    public void loadData(InitData initData) {
        resumeNameTextField.setText(initData.getName());
        remarkTextField.setText(initData.getRemark());
    }

    @Override
    public void initialize() {
        resumeNameTextField.setText(RandomUtil.randomString(Constants.sample, 5));
        remarkTextField.setText(RandomUtil.randomString(Constants.sample, 15));
    }

    @Data
    @EqualsAndHashCode(callSuper = false)
    public static class InitData extends CommunicativeController.InitData {
        private String name;
        private String remark;
    }

    @FXML
    private VBox rootPane;

    @FXML
    private JFXTextField resumeNameTextField;

    @FXML
    private JFXTextField remarkTextField;

    @FXML
    private Text fileNameText;

    private File resumeFile;

    @FXML
    void addFile(ActionEvent event) {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("告诉我在哪？");
        File chooseFile = fileChooser.showOpenDialog(rootPane.getScene().getWindow());

        if (chooseFile != null) {
            String fileType = FileUtil.getType(chooseFile);
            switch (fileType) {
                case "png":
                case "jpg":
                case "jpeg":
                case "webm":
                case "pdf":
                case "doc":
                case "docx":
                    fileNameText.setText(chooseFile.getName());
                    resumeFile = chooseFile;
                    break;
                default:
                    dialogMaker.createMessageDialog("（；´д｀）ゞ", "简历文件不支持：" + fileType + "\n仅支持png, jpg, webm, pdf, word");
            }
        }
    }

    public String getResumeName() {
        return resumeNameTextField.getText();
    }

    public String getRemark() {
        return remarkTextField.getText();
    }

    public File getResumeFile() {
        return resumeFile;
    }
}
