package net.lensfrex.disillusion.client.controller.page.employer;

import cn.hutool.json.JSONUtil;
import com.jfoenix.effects.JFXDepthManager;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.ScrollPane;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.VBox;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.controller.components.employer.application.ApplicationOfferCard;
import net.lensfrex.disillusion.client.controller.components.student.resume.ResumeCard;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.util.List;

@Slf4j
public class EmployerApplicationManagePage extends VBox {
    @FXML
    private VBox rootPane;

    @FXML
    private ScrollPane scrollPane;

    @FXML
    private FlowPane offerContainer;

    private String token;
    private ThreadSafeDialogMaker dialogMaker;

    @FXML
    void refresh(ActionEvent event) {
        this.refreshOfferList();
    }

    public void loadData(String token, ThreadSafeDialogMaker dialogMaker) {
        this.token = token;
        this.dialogMaker = dialogMaker;
        this.refreshOfferList();
    }

    public void initialize() {
        offerContainer.getChildren().addAll(offerLost);
    }

    private final ObservableList<ResumeCard> offerLost = FXCollections.observableArrayList();

    private void refreshOfferList() {
        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_GET_BY_EMPLOYER_SELF_URL)
                .token(token)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "获取offer列表时发生了网络错误\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "获取offer列表不成功\n" + s))
                        .successCallback(data -> {
                            List<OfferResponse> result = JSONUtil.toList(data, OfferResponse.class);
                            Platform.runLater(() -> showOffers(result));
                        })
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void showOffers(List<OfferResponse> offers) {
        try {
            ObservableList<Node> cards = offerContainer.getChildren();
            cards.clear();
            for (OfferResponse offer : offers) {
                if (offer == null) {
                    continue;
                }
                ApplicationOfferCard card =
                        ComponentFactory.loadComponent("fxml/employer/application/applicationOfferCard.fxml", new ApplicationOfferCard(offer));
                card.loadData(token, dialogMaker);
                JFXDepthManager.setDepth(card, 1);
                cards.add(card);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.warn("展示offer卡片时发生错误");
        }
    }
}
