package net.lensfrex.disillusion.client.controller.page.employer;

import animatefx.animation.FadeIn;
import animatefx.animation.FadeOut;
import cn.hutool.json.JSONUtil;
import javafx.application.Platform;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.image.ImageView;
import javafx.scene.layout.Pane;
import javafx.scene.layout.StackPane;
import javafx.scene.text.Text;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.controller.page.student.StudentProfilePage;
import net.lensfrex.disillusion.client.data.response.EmployerInfoResponse;
import net.lensfrex.disillusion.client.util.DialogMaker;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.util.HashMap;
import java.util.Map;

@Slf4j
public class EmployerMainController {
    @FXML
    private StackPane rootPane;

    @FXML
    private ImageView avatarImageView;

    @FXML
    private Text employerName;

    @FXML
    private Text usernameText;

    @FXML
    private Text shortDescribeText;

    @FXML
    private StackPane containerPane;

    private ThreadSafeDialogMaker threadSafeDialogMaker;

    private String token;

    private EmployerInfoResponse employerInfo;

    private final Map<String, Pane> pages = new HashMap<>(4);

    public void setData(String token) {
        this.token = token;
        this.fetchUserInfo();
    }

    public void initialize() {
        threadSafeDialogMaker = new ThreadSafeDialogMaker(rootPane);
        try {
            pages.put("profile", ComponentFactory.loadComponent("fxml/employer/profile.fxml", EmployerProfilePage.class));
            pages.put("offerManage", ComponentFactory.loadComponent("fxml/employer/offer/offerManagePage.fxml", EmployerOfferManagePage.class));
            pages.put("applicationManage", ComponentFactory.loadComponent("fxml/employer/application/applicationManagePage.fxml", EmployerApplicationManagePage.class));
        } catch (Exception e) {
            e.printStackTrace();
            DialogMaker.errorMessageDialog("出错啦，请联系lensfrex这家伙\n调试信息：\n" + e);
        }
    }

    private void fetchUserInfo() {
        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_INFO_VIEW_URL)
                .token(token)
                .failCallback(e -> log.error("请求用户数据时发生网络错误"))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> threadSafeDialogMaker.createMessageDialog("啊哈", "获取用户信息不成功：" + s))
                        .successCallback(this::updateUserInfoUI)
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void updateUserInfoUI(String data) {
        employerInfo = JSONUtil.toBean(data, EmployerInfoResponse.class);
        Platform.runLater(() -> {
            employerName.setText(employerInfo.getName());
            usernameText.setText("@" + employerInfo.getId());
            shortDescribeText.setText(employerInfo.getShortDescribe());
        });

        this.updateAvatar();
    }

    private void updateAvatar() {
        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_AVATAR_VIEW_URL)
                .token(token)
                .failCallback(e -> log.warn("获取头像时发生网络错误：\n" + e))
                .successCallback(avatarResponse -> Platform.runLater(() -> StudentProfilePage.setAvatar(avatarImageView, avatarResponse)))
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    @FXML
    void showApplicationDealPage(ActionEvent event) {
        EmployerApplicationManagePage page = (EmployerApplicationManagePage) this.switchPage("applicationManage");
        page.loadData(token, threadSafeDialogMaker);
    }

    @FXML
    void showOfferManagePage(ActionEvent event) {
        EmployerOfferManagePage page = (EmployerOfferManagePage) this.switchPage("offerManage");
        page.loadData(token, threadSafeDialogMaker);
    }

    @FXML
    void showProfile(ActionEvent event) {
        EmployerProfilePage page = (EmployerProfilePage) this.switchPage("profile");
        page.loadData(employerInfo, token, threadSafeDialogMaker, r -> fetchUserInfo());
    }

    private Node switchPage(String page) {
        ObservableList<Node> list = containerPane.getChildren();

        new FadeOut(list.get(0)).play();
        Pane pagePane = pages.get(page);
        if (pagePane != null) {
            list.clear();
            list.add(pagePane);
            new FadeIn(pagePane).play();
        }

        return pagePane;
    }
}