package net.lensfrex.disillusion.client.controller.page.employer;

import cn.hutool.json.JSONUtil;
import com.jfoenix.effects.JFXDepthManager;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.ScrollPane;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.VBox;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.controller.components.employer.offer.EmployerOfferCard;
import net.lensfrex.disillusion.client.controller.components.employer.offer.EmployerOfferDetailPane;
import net.lensfrex.disillusion.client.controller.components.student.resume.ResumeCard;
import net.lensfrex.disillusion.client.data.request.OfferPublishRequest;
import net.lensfrex.disillusion.client.data.response.OfferResponse;
import net.lensfrex.disillusion.client.util.Consumer;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;

import java.io.IOException;
import java.util.List;

@Slf4j
public class EmployerOfferManagePage extends VBox {
    @FXML
    private VBox rootPane;

    @FXML
    private ScrollPane scrollPane;

    @FXML
    private FlowPane offerContainer;

    private String token;
    private ThreadSafeDialogMaker dialogMaker;

    public void loadData(String token, ThreadSafeDialogMaker dialogMaker) {
        this.token = token;
        this.dialogMaker = dialogMaker;
        this.refreshOfferList();
    }

    public void initialize() {
        offerContainer.getChildren().addAll(offerLost);
    }

    private final ObservableList<ResumeCard> offerLost = FXCollections.observableArrayList();

    @FXML
    void addOffer(ActionEvent event) throws IOException, InstantiationException, IllegalAccessException {
        EmployerOfferDetailPane offerDetailPane =
                ComponentFactory.loadComponent("fxml/employer/offer/offerDetailPane.fxml", EmployerOfferDetailPane.class);
        dialogMaker.createDialogWithOKAndCancel("发布新的offer信息", offerDetailPane, e -> this.publishOffer(offerDetailPane.getData()));
    }

    private void publishOffer(OfferPublishRequest offerPublishRequest) {
        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_PUBLISH_URL)
                .token(token)
                .body(offerPublishRequest)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "发布offer时发生了网络问题：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "发布offer不成功：\n" + s))
                        .successCallback(s -> this.refreshOfferList())
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.POST);
    }

    private void refreshOfferList() {
        new OkhttpUtil()
                .url(Constants.OfferRequestUrls.OFFER_GET_BY_EMPLOYER_SELF_URL)
                .token(token)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "获取offer列表时发生了网络错误\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "获取offer列表不成功\n" + s))
                        .successCallback(data -> {
                            List<OfferResponse> result = JSONUtil.toList(data, OfferResponse.class);
                            Platform.runLater(() -> showOffers(result));
                        })
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void showOffers(List<OfferResponse> offers) {
        try {
            ObservableList<Node> cards = offerContainer.getChildren();
            cards.clear();
            for (OfferResponse offer : offers) {
                if (offer == null) {
                    continue;
                }
                EmployerOfferCard card =
                        ComponentFactory.loadComponent("fxml/employer/offer/offerCard.fxml", new EmployerOfferCard(offer));
                card.loadData(token, dialogMaker, this.deleteCallback, this.updateCallback);
                JFXDepthManager.setDepth(card, 1);
                cards.add(card);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.warn("展示offer卡片时发生错误");
        }
    }

    private final Consumer<String> updateCallback = s -> refreshOfferList();

    private final Consumer<String> deleteCallback = s -> refreshOfferList();
}
