package net.lensfrex.disillusion.client.controller.page.student;

import cn.hutool.json.JSONUtil;
import com.jfoenix.effects.JFXDepthManager;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.scene.Node;
import javafx.scene.control.ScrollPane;
import javafx.scene.layout.FlowPane;
import javafx.scene.layout.VBox;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.controller.CommunicativeController;
import net.lensfrex.disillusion.client.controller.components.ComponentFactory;
import net.lensfrex.disillusion.client.controller.components.student.application.ApplicationCard;
import net.lensfrex.disillusion.client.data.response.ApplicationResponse;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;
import okhttp3.*;

import java.util.List;

@Slf4j
public class ApplicationManagePage extends VBox implements CommunicativeController<ApplicationManagePage.InitData> {
    @FXML
    private VBox rootPane;

    @FXML
    private ScrollPane scrollPane;

    @FXML
    private FlowPane applicationContainer;

    private String token;
    private ThreadSafeDialogMaker dialogMaker;

    private static final OkHttpClient okhttpClient = Constants.globalClient;

    @Data
    @Builder
    @EqualsAndHashCode(callSuper = false)
    public static class InitData extends CommunicativeController.InitData {
        private String token;
        private ThreadSafeDialogMaker dialogMaker;
    }

    @Override
    public void loadData(ApplicationManagePage.InitData initData) {
        this.token = initData.token;
        this.dialogMaker = initData.dialogMaker;

        this.refreshApplicationList();
    }

    public void initialize() {
        applicationContainer.getChildren().addAll(applicationList);
    }

    private final ObservableList<ApplicationCard> applicationList = FXCollections.observableArrayList();

    private void refreshApplicationList() {
        new OkhttpUtil()
                .url(Constants.ApplicationRequestUrls.APPLICATION_STUDENT_VIEW_URL)
                .token(token)
                .failCallback(e ->  dialogMaker.createMessageDialog("啊哈", "获取简历时发生了网络错误\n" + e))
                .successCallback(response -> {
                    ResponseHandler.Result responseData = ResponseHandler.parse(response,
                            s -> dialogMaker.createMessageDialog("啊哈", "获取简历时出错了...\n" + s));
                    if (responseData.data == null || !responseData.success) {
                        return;
                    }

                    List<ApplicationResponse> result = JSONUtil.toList(responseData.data, ApplicationResponse.class);
                    Platform.runLater(() -> listApplications(result));
                })
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    private void listApplications(List<ApplicationResponse> applications) {
        try {
            ObservableList<Node> items = applicationContainer.getChildren();
            items.clear();
            for (ApplicationResponse application : applications) {
                if (application == null) {
                    continue;
                }

                ApplicationCard card =
                        ComponentFactory.loadComponent("fxml/student/application/applicationCard.fxml",
                                new ApplicationCard(token, dialogMaker, application, s -> this.refreshApplicationList()));
                JFXDepthManager.setDepth(card, 1);
                card.setUserData(applications);
                items.add(card);
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.warn("展示求职申请卡片时发生错误");
        }
    }
}
