package net.lensfrex.disillusion.client.controller.page.student;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.io.FileUtil;
import com.jfoenix.controls.JFXRadioButton;
import com.jfoenix.controls.JFXTextArea;
import com.jfoenix.controls.JFXTextField;
import javafx.application.Platform;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.scene.control.ToggleGroup;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;
import javafx.scene.text.Text;
import javafx.stage.FileChooser;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.config.Constants;
import net.lensfrex.disillusion.client.data.response.StudentInfoResponse;
import net.lensfrex.disillusion.client.util.ImageUtil;
import net.lensfrex.disillusion.client.util.OkhttpUtil;
import net.lensfrex.disillusion.client.util.ResponseHandler;
import net.lensfrex.disillusion.client.util.ThreadSafeDialogMaker;
import okhttp3.Response;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.function.Consumer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class StudentProfilePage extends HBox {
    @FXML
    private HBox rootPane;

    @FXML
    private ImageView avatarImageView;

    @FXML
    private Text usernameText;

    @FXML
    private Text signText;

    @FXML
    private JFXTextField signEditText;

    @FXML
    private JFXTextArea describeTextArea;

    @FXML
    private JFXTextField realNameTextField;

    @FXML
    private JFXTextField ageTextField;

    @FXML
    private JFXRadioButton maleRadioButton;

    @FXML
    private JFXRadioButton femaleRadioButton;

    @FXML
    private JFXTextField contactTextField;

    @FXML
    private JFXTextField idNumberTextField;

    private File newAvatarFile;

    private String token;

    private static final Pattern agePattern = Pattern.compile("^\\d{2}$");

    private ThreadSafeDialogMaker dialogMaker;

    @FXML
    void save(ActionEvent event) {
        studentInfo.setRealName(realNameTextField.getText());
        studentInfo.setShortDescribe(signEditText.getText());
        studentInfo.setDescribe(describeTextArea.getText());
        studentInfo.setPhoneNumber(contactTextField.getText());
        studentInfo.setIdNumber(idNumberTextField.getText());

        String inputAge = ageTextField.getText();
        Matcher ageMatcher = agePattern.matcher(inputAge);
        if (!ageMatcher.matches()) {
            dialogMaker.createMessageDialog("啊哈", "年龄格式不正确（2位数字）");
        }

        studentInfo.setAge(Integer.parseInt(inputAge));

        studentInfo.setSex(sexGroup.getSelectedToggle().getUserData().equals("0") ? 0 : 1);

        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_INFO_EDIT_URL)
                .token(token)
                .body(studentInfo)
                .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "更新上传用户信息的时候出错啦：\n" + e))
                .successCallback(response -> new ResponseHandler(response)
                        .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "更新上传用户信息不成功：" + s))
                        .successCallback(s -> {
                            updateCallback.accept(studentInfo);
                            dialogMaker.createMessageDialog("好耶", "更新成功");
                        })
                        .process())
                .executeAsync(OkhttpUtil.RequestMethod.POST);
    }

    @FXML
    void replaceAvatar(MouseEvent event) {
        FileChooser fileChooser = new FileChooser();
        fileChooser.setTitle("选择图片");
        File chooseFile = fileChooser.showOpenDialog(rootPane.getScene().getWindow());

        if (chooseFile != null) {
            String fileType = FileUtil.getType(chooseFile);
            switch (fileType) {
                case "png":
                case "jpg":
                case "jpeg":
                case "gif":
                case "webm":
                    this.uploadAvatar(chooseFile);
                    break;
                default:
                    dialogMaker.createMessageDialog(
                            "（；´д｀）ゞ",
                            "头像文件不支持：" + fileType + "\n仅支持png, jpg, gif, webm"
                    );
            }
        }
    }

    private void uploadAvatar(File avatarFile) {
        try {
            byte[] avatarData;
            // 大于1m的图片就进行压缩
            long fileLength = avatarFile.length();
            if (fileLength > 1048576) {
                // 压缩比例，文件越大压缩程度越大
                float percent = (float) Math.pow(0.96, fileLength / 1048576f);
                avatarData = ImageUtil.processImage(avatarFile, 0.8f, percent, "jpg");
            } else {
                avatarData = Files.readAllBytes(Paths.get(avatarFile.toURI()));
            }

            new OkhttpUtil()
                    .url(Constants.UserRequestUrls.USER_AVATAR_EDIT_URL)
                    .token(token)
                    .body(Base64.encode(avatarData), "plain/text")
                    .failCallback(e -> dialogMaker.createMessageDialog("啊哈", "上传头像文件时发生网络错误：" + e))
                    .successCallback(response -> new ResponseHandler(response)
                            .errorCallback(s -> dialogMaker.createMessageDialog("啊哈", "获取头像时出现问题：" + s))
                            .successCallback(s -> {
                                updateCallback.accept(studentInfo);
                                Platform.runLater(() -> setAvatar(avatarImageView, avatarData));
                            })
                            .process())
                    .executeAsync(OkhttpUtil.RequestMethod.POST);
        } catch (IOException e) {
            dialogMaker.createMessageDialog("啊哈", "读取头像文件时发生错误：" + e);
        }
    }

    private final ToggleGroup sexGroup = new ToggleGroup();

    @FXML
    public void initialize() {
        maleRadioButton.setUserData("0");
        femaleRadioButton.setUserData("1");

        maleRadioButton.setToggleGroup(sexGroup);
        femaleRadioButton.setToggleGroup(sexGroup);
    }

    private StudentInfoResponse studentInfo;
    private Consumer<StudentInfoResponse> updateCallback;

    public void setData(StudentInfoResponse studentInfo,
                        String token, ThreadSafeDialogMaker dialogMaker,
                        Consumer<StudentInfoResponse> updateCallback) {

        this.studentInfo = studentInfo;
        this.token = token;
        this.dialogMaker = dialogMaker;
        this.updateCallback = updateCallback;

        usernameText.setText(studentInfo.getId());
        realNameTextField.setText(studentInfo.getRealName());
        signText.setText(studentInfo.getShortDescribe());
        signEditText.setText(studentInfo.getShortDescribe());
        describeTextArea.setText(studentInfo.getDescribe());
        contactTextField.setText(studentInfo.getPhoneNumber());
        idNumberTextField.setText(studentInfo.getIdNumber());
        ageTextField.setText(studentInfo.getAge() + "");

        switch (studentInfo.getSex()) {
            case 0:
                sexGroup.selectToggle(maleRadioButton);
                break;
            case 1:
                sexGroup.selectToggle(femaleRadioButton);
                break;
        }

        new OkhttpUtil()
                .url(Constants.UserRequestUrls.USER_AVATAR_VIEW_URL)
                .token(token)
                .successCallback(response -> Platform.runLater(() -> setAvatar(avatarImageView, response)))
                .executeAsync(OkhttpUtil.RequestMethod.GET);
    }

    public static void setAvatar(ImageView avatarImageView, Response response) {
        try {
            assert response.body() != null;
            byte[] data = response.body().bytes();

            setAvatar(avatarImageView, data);
        } catch (Exception e) {
            log.warn("获取头像数据时出现异常", e);
        }
    }

    public static void setAvatar(ImageView avatarImageView, byte[] data) {
        Image avatarImage = new Image(new ByteArrayInputStream(data));
        avatarImageView.setImage(avatarImage);
    }
}