package net.lensfrex.disillusion.client.util;

import com.jfoenix.controls.JFXButton;
import com.jfoenix.controls.JFXDialog;
import com.jfoenix.controls.JFXDialogLayout;
import javafx.beans.NamedArg;
import javafx.event.ActionEvent;
import javafx.event.EventHandler;
import javafx.geometry.Pos;
import javafx.scene.Node;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.layout.HBox;
import javafx.scene.layout.Pane;
import javafx.scene.layout.StackPane;
import javafx.scene.paint.Paint;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;
import javafx.scene.text.Text;
import javafx.stage.Stage;

public class DialogMaker {
    Pane rootPane;

    public DialogMaker(@NamedArg("rootPane") Pane rootPane) {
        this.rootPane = rootPane;
    }

    public void createMessageDialog(@NamedArg("title") String title, @NamedArg("message") String message) {
        JFXButton OKButton = new JFXButton("了解！");
        OKButton.setFont(Font.font("Microsoft YaHei", FontWeight.BOLD, 12));
        OKButton.setPrefWidth(60);
        OKButton.setPrefHeight(30);

        Text messageText = new Text(message);
        messageText.setFont(Font.font("Microsoft YaHei", 14));

        createDialog(title, messageText, OKButton);
    }

    //创建只有一个按钮的dialog
    public void createDialogWithOneBtn(@NamedArg("title") String title, @NamedArg("theBody") Node body, @NamedArg("OKEvent") EventHandler<ActionEvent> event) {
        //dialog.setPrefHeight(rootPane.getPrefHeight());
        //dialog.setPrefWidth(rootPane.getPrefWidth());

        JFXButton OKButton = new JFXButton("好的！");
        OKButton.setFont(Font.font("Microsoft YaHei", FontWeight.BOLD, 12));
        OKButton.setPrefWidth(60);
        OKButton.setPrefHeight(30);

        createDialog(title, body, OKButton);

        if (event != null) {
            OKButton.addEventHandler(ActionEvent.ACTION, event);
        }
    }

    public void createDialogWithOneBtn(@NamedArg("title") String title, @NamedArg("theBody") Node body) {
        createDialogWithOneBtn(title, body, null);
    }

    //创建有OK和cancel按钮的dialog
    public void createDialogWithOKAndCancel(@NamedArg("title") String title, @NamedArg("message") String message, @NamedArg("OKEvent") EventHandler<ActionEvent> OKEvent) {
        Text messageText = new Text(message);
        messageText.setFont(Font.font("Microsoft YaHei", 14));

        createDialogWithOKAndCancel(title, messageText, OKEvent);
    }

    //创建有OK和cancel按钮的dialog
    public void createDialogWithOKAndCancel(@NamedArg("title") String title, @NamedArg("body") Node body, @NamedArg("OKEvent") EventHandler<ActionEvent> OKEvent) {
        JFXButton CancelButton = new JFXButton("手滑了");
        CancelButton.setFont(Font.font("Microsoft YaHei", FontWeight.BOLD, 12));
        CancelButton.setPrefWidth(60);
        CancelButton.setPrefHeight(30);

        JFXButton OKButton = new JFXButton("是！");
        OKButton.setFont(Font.font("Microsoft YaHei", FontWeight.BOLD, 12));
        OKButton.setPrefWidth(60);
        OKButton.setPrefHeight(30);
        OKButton.setTextFill(Paint.valueOf("red"));
        OKButton.addEventHandler(ActionEvent.ACTION, OKEvent);

        createDialog(title, body, CancelButton, OKButton);
    }

    public void createDialog(@NamedArg("title") String title, @NamedArg("theBody") Node body, @NamedArg("buttons") JFXButton... buttons) {

        JFXDialogLayout content = new JFXDialogLayout();

        Label titleLabel = new Label(title);
        titleLabel.setFont(Font.font("Microsoft YaHei", FontWeight.BOLD, 22));
        content.setHeading(titleLabel);

        content.setBody(body);
        content.setAlignment(Pos.CENTER);

        StackPane tempPane = new StackPane();
        tempPane.setPrefHeight(rootPane.getPrefHeight());
        tempPane.setPrefWidth(rootPane.getPrefWidth());
        rootPane.getChildren().add(tempPane);

        JFXDialog dialog = new JFXDialog(tempPane, content, JFXDialog.DialogTransition.TOP);

        dialog.setOnDialogClosed(event -> rootPane.getChildren().remove(tempPane));

        for (JFXButton button : buttons) {
            button.addEventHandler(ActionEvent.ACTION, e -> dialog.close());
        }

        content.setActions(buttons);

        dialog.show();
    }


    public static void errorMessageDialog(String message) {
        HBox box = new HBox(new TextArea(message));
        box.setAlignment(Pos.CENTER);

        Scene scene = new Scene(box, 500, 350);

        Stage stage = new Stage();
        stage.setTitle("出事了");
        stage.setScene(scene);
        stage.show();
    }

}
