package net.lensfrex.disillusion.client.util;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.lensfrex.disillusion.client.data.ResponseCode;
import okhttp3.Response;

import java.io.IOException;

@Slf4j
public class ResponseHandler {
    @SafeVarargs
    public static Result parse(Response response, Consumer<String>... errorCallbacks) throws IOException {
        assert response.body() != null;
        String responseString = response.body().string();
        JSONObject responseJson = JSONUtil.parseObj(responseString);

        boolean success = true;
        Integer code = responseJson.getInt("code");
        String message = responseJson.getStr("message");
        if (code == null || !code.equals(ResponseCode.SUCCESS.getCode())) {
            if (message == null) {
                message = responseString;
            }
            if (errorCallbacks != null) {
                for (Consumer<String> errorCallback : errorCallbacks) {
                    errorCallback.accept(message);
                }
            }

            success = false;
            log.warn("请求错误：" + message);
        }

        return new Result(success, message, responseJson.getStr("data"));
    }

    private final Response response;

    private Consumer<String> errorCallback;
    private Consumer<String> successCallback;

    public ResponseHandler(Response response) {
        this.response = response;
    }

    public ResponseHandler errorCallback(Consumer<String> errorCallback) {
        this.errorCallback = errorCallback;
        return this;
    }

    public ResponseHandler successCallback(Consumer<String> successCallback) {
        this.successCallback = successCallback;
        return this;
    }

    public void process() throws IOException {
        Result result = parse(response, errorCallback);
        if (result.success) {
            successCallback.accept(result.data);
        }
    }

    @AllArgsConstructor
    public static class Result {
        public final boolean success;
        public final String message;
        public final String data;
    }
}
