package net.lensfrex.disillusion.util;

import com.auth0.jwt.JWT;
import com.auth0.jwt.JWTVerifier;
import com.auth0.jwt.algorithms.Algorithm;
import com.auth0.jwt.exceptions.JWTVerificationException;
import net.lensfrex.disillusion.configure.GlobalConfigure;
import org.springframework.stereotype.Component;

import java.time.Instant;
import java.time.temporal.ChronoUnit;

@Component
public class TokenUtil {

    public TokenUtil(GlobalConfigure globalConfigure) {
    }

    public static String signNewToken(String user) {
        String secret = GlobalConfigure.getJwtSecret();
        return JWT.create()
                .withClaim("user", user)
                .withClaim("issue", Instant.now().getNano())
                .withExpiresAt(Instant.now().plus(30, ChronoUnit.DAYS))
                .sign(Algorithm.HMAC256(secret));
    }

    public static boolean verify(String token) {
        String secret = GlobalConfigure.getJwtSecret();
        JWTVerifier verifier = JWT.require(Algorithm.HMAC256(secret)).build();
        try {
            verifier.verify(token);
        } catch (JWTVerificationException e) {
            return false;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }

        return true;
    }

    public static String getUsernameFromToken(String token) {
        return JWT.decode(token).getClaim("user").asString();
    }
}
