package net.lensfrex.disillusion.web.controller;

import net.lensfrex.disillusion.annotation.CheckAuth;
import net.lensfrex.disillusion.model.User;
import net.lensfrex.disillusion.model.student.Resume;
import net.lensfrex.disillusion.response.general.Response;
import net.lensfrex.disillusion.response.general.ResponseCode;
import net.lensfrex.disillusion.util.TokenUtil;
import net.lensfrex.disillusion.web.service.student.ResumeService;
import net.lensfrex.disillusion.web.service.user.UserService;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;

@RestController
@RequestMapping("/resume")
public class ResumeController {
    private final ResumeService resumeService;

    private final UserService userService;

    public ResumeController(ResumeService resumeService,
                            UserService userService) {
        this.resumeService = resumeService;
        this.userService = userService;
    }

    @CheckAuth
    @GetMapping("/view")
    public Response<Resume> view(@RequestHeader("token") String token, @RequestParam("id") String id)
            throws IOException, ClassNotFoundException {

        Resume result = resumeService.getResume(id);

        String username = TokenUtil.getUsernameFromToken(token);
        User user = userService.getAccount(username);
        if (result == null) {
            return Response.success();
        }

        if (result.getOwnerId().equals(username) || user.getUserType() == User.UserType.EMPLOYER) {
            return Response.success(result);
        } else {
            return Response.error(ResponseCode.PERMISSION_DENIED);
        }
    }
}
