package net.lensfrex.disillusion.web.controller.employer;

import net.lensfrex.disillusion.annotation.CheckAuth;
import net.lensfrex.disillusion.model.OfferApplication;
import net.lensfrex.disillusion.model.User;
import net.lensfrex.disillusion.model.employer.Offer;
import net.lensfrex.disillusion.response.general.Response;
import net.lensfrex.disillusion.response.general.ResponseCode;
import net.lensfrex.disillusion.web.request.OfferPublishRequest;
import net.lensfrex.disillusion.web.service.employer.OfferService;
import net.lensfrex.disillusion.web.service.student.ApplicationService;
import net.lensfrex.disillusion.web.service.user.UserService;
import net.lensfrex.disillusion.util.TokenUtil;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.Arrays;

@RestController
@RequestMapping("/employer/offer")
public class EmolpyerOfferController {

    private final UserService userService;

    private final OfferService offerService;

    private final ApplicationService applicationService;

    public EmolpyerOfferController(UserService userService,
                                   OfferService offerService,
                                   ApplicationService applicationService) {

        this.userService = userService;
        this.offerService = offerService;
        this.applicationService = applicationService;
    }

    @CheckAuth
    @PostMapping("/publish")
    public Response<Object> publish(@RequestBody OfferPublishRequest offerRequest, @RequestHeader("token") String token)
            throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        User user = userService.getAccount(username);
        if (user.getUserType() != User.UserType.EMPLOYER) {
            return Response.error(ResponseCode.PERMISSION_DENIED);
        }

        offerService.addOffer(username, offerRequest);
        return Response.success();
    }

    @CheckAuth
    @PostMapping("/delete")
    public Response<Object> delete(@RequestHeader("token") String token, @RequestParam("id") String id)
            throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        if (!check(username, id)) {
            return Response.error(ResponseCode.PERMISSION_DENIED);
        }

        offerService.deleteOffer(username, id);

        return Response.success();
    }

    @CheckAuth
    @GetMapping("/view")
    public Response<Offer[]> getAllOffer(@RequestHeader("token") String token)
            throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        Offer[] offers = offerService.getAllOfferByUser(username);

        return Response.success(offers);
    }

    @CheckAuth
    @PostMapping("/update")
    public Response<Object> update(@RequestHeader("token") String token, @RequestParam("id") String id, @RequestBody OfferPublishRequest offerRequest)
            throws IOException, ClassNotFoundException {

        String user = TokenUtil.getUsernameFromToken(token);
        if (!check(user, id)) {
            return Response.error(ResponseCode.PERMISSION_DENIED);
        }

        offerService.updateOffer(user, id, offerRequest);

        return Response.success();
    }

    private boolean check(String user, String id) throws IOException, ClassNotFoundException {
        Offer offer = offerService.getOffer(id);

        return offer != null && offer.getPublisherId().equals(user);
    }
}
