package net.lensfrex.disillusion.web.controller.employer;

import net.lensfrex.disillusion.annotation.CheckAuth;
import net.lensfrex.disillusion.model.OfferApplication;
import net.lensfrex.disillusion.model.User;
import net.lensfrex.disillusion.response.general.Response;
import net.lensfrex.disillusion.response.general.ResponseCode;
import net.lensfrex.disillusion.util.TokenUtil;
import net.lensfrex.disillusion.web.service.student.ApplicationService;
import net.lensfrex.disillusion.web.service.user.UserService;
import org.springframework.beans.BeanUtils;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.util.Arrays;

@RestController
@RequestMapping("/employer/application")
public class EmployerOfferApplicationController {
    private final UserService userService;

    private final ApplicationService applicationService;

    public EmployerOfferApplicationController(UserService userService,
                                              ApplicationService applicationService) {

        this.userService = userService;
        this.applicationService = applicationService;
    }

    @CheckAuth
    @PostMapping("/deal")
    public Response<Object> deal(@RequestHeader("token") String token, @RequestParam("id") String id, @RequestParam("opinion") String opinion) throws IOException, ClassNotFoundException {
        String username = TokenUtil.getUsernameFromToken(token);
        User user = userService.getAccount(username);
        if (user.getUserType() != User.UserType.EMPLOYER) {
            return Response.error(ResponseCode.PERMISSION_DENIED);
        }

        OfferApplication application = applicationService.getOfferApplication(id);

        if (!application.getTargetId().equals(username)) {
            return Response.error(ResponseCode.PERMISSION_DENIED);
        }

        OfferApplication.Status status = switch (opinion) {
            case "accept" -> OfferApplication.Status.ACCEPTED;
            case "reject" -> OfferApplication.Status.REJECTED;
            default -> throw new IllegalStateException();
        };

        applicationService.changeApplicationStatus(id, status);

        return Response.success();
    }

    @CheckAuth
    @GetMapping("/view")
    public Response<OfferApplication[]> view(@RequestHeader("token") String token, @RequestParam("offer_id") String offerId)
            throws IOException, ClassNotFoundException {

        OfferApplication[] applications = applicationService.getAllApplicationByOfferId(offerId);
        OfferApplication[] result = new OfferApplication[applications.length];
        int count = 0;
        for (OfferApplication application : applications) {
            if (application != null && application.getStatus() != OfferApplication.Status.ACCEPTED) {
                result[count] = new OfferApplication();
                BeanUtils.copyProperties(application, result[count]);
                count++;
            }
        }

        return Response.success(Arrays.copyOf(result, count));
    }
}
