package net.lensfrex.disillusion.web.controller.student;

import net.lensfrex.disillusion.annotation.CheckAuth;
import net.lensfrex.disillusion.model.User;
import net.lensfrex.disillusion.model.student.Resume;
import net.lensfrex.disillusion.response.general.Response;
import net.lensfrex.disillusion.response.general.ResponseCode;
import net.lensfrex.disillusion.util.TokenUtil;
import net.lensfrex.disillusion.web.request.ResumeRequest;
import net.lensfrex.disillusion.web.service.student.ResumeService;
import net.lensfrex.disillusion.web.service.user.UserService;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;

@RestController
@RequestMapping("/student/resume")
public class StudentResumeController {
    private final ResumeService resumeService;
    private final UserService userService;

    public StudentResumeController(ResumeService resumeService, UserService userService) {
        this.resumeService = resumeService;
        this.userService = userService;
    }

    @CheckAuth
    @PostMapping(value = "/add")
    public Response<Object> add(@RequestBody ResumeRequest request, @RequestHeader("token") String token)
            throws IOException, ClassNotFoundException {

        String user = TokenUtil.getUsernameFromToken(token);
        resumeService.addResume(user, request);

        return Response.success();
    }

    @CheckAuth
    @PostMapping(value = "/update")
    public Response<Object> update(@RequestBody ResumeRequest resumeRequest,
                                   @RequestParam("id") String id, @RequestHeader("token") String token)
            throws IOException, ClassNotFoundException {

        String user = TokenUtil.getUsernameFromToken(token);
        if (!check(user, id)) {
            return Response.error(ResponseCode.PERMISSION_DENIED);
        }
        resumeService.updateResume(user, id, resumeRequest);

        return Response.success();
    }

    @CheckAuth
    @PostMapping("/delete")
    public Response<Object> deleteResume(@RequestParam("id") String id, @RequestHeader("token") String token)
            throws IOException, ClassNotFoundException {

        String user = TokenUtil.getUsernameFromToken(token);
        if (!check(user, id)) {
            return Response.error(ResponseCode.PERMISSION_DENIED);
        }

        resumeService.deleteResume(user, id);

        return Response.success();
    }

    @CheckAuth
    @GetMapping("/all")
    public Response<Resume[]> getAllResume(@RequestHeader("token") String token)
            throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        Resume[] result = resumeService.getAllResume(username);

        return Response.success(result);
    }

    @CheckAuth
    @GetMapping("/getFile")
    public Response<String> getFile(@RequestParam("id") String id, @RequestHeader("token") String token)
            throws IOException, ClassNotFoundException {

        String username = TokenUtil.getUsernameFromToken(token);
        if (!check(username, id)) {
            User user = userService.getAccount(username);
            if (user == null || user.getUserType() != User.UserType.EMPLOYER) {
                return Response.error(ResponseCode.PERMISSION_DENIED);
            }
        }

        String fileData = resumeService.getResumeFileData(id);

        return Response.success(fileData);
    }

    private boolean check(String user, String id) throws IOException, ClassNotFoundException {
        Resume resume = resumeService.getResume(id);
        return resume != null && resume.getOwnerId().equals(user);
    }
}
