package net.lensfrex.disillusion.web.interceptor;

import com.fasterxml.jackson.databind.ObjectMapper;
import net.lensfrex.disillusion.annotation.CheckAuth;
import net.lensfrex.disillusion.response.general.Response;
import net.lensfrex.disillusion.response.general.ResponseCode;
import net.lensfrex.disillusion.util.TokenUtil;
import org.springframework.stereotype.Service;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import java.io.PrintWriter;

/**
 * token验证拦截器
 */
@Service
public class TokenInterceptor implements HandlerInterceptor {
    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (!needCheckAuth(handler)) {
            return true;
        }

        String token = request.getHeader("token");
        if (token == null || !TokenUtil.verify(token)) {
            response.setCharacterEncoding("utf-8");
            response.setContentType("application/json;charset=utf-8");
            PrintWriter writer = response.getWriter();

            String result = new ObjectMapper().writeValueAsString(
                    Response.error(ResponseCode.TOKEN_INVALID));
            writer.print(result);
            writer.close();

            return false;
        }

        return true;
    }

    private boolean needCheckAuth(Object handler) {
        if (handler instanceof HandlerMethod handlerMethod) {
            CheckAuth annotation = handlerMethod.getMethod().getAnnotation(CheckAuth.class);
            return annotation != null;
        }

        return false;
    }
}