package net.lensfrex.disillusion.data;

import java.io.Serializable;

// 极其简陋的hashMap实现
public class Hash<K, V> implements Serializable {
    private static final int TABLE_SIZE = 32;

    private final Node<K, V>[] table;

    private int size = 0;

    @SuppressWarnings({"unchecked"})
    public Hash() {
        this.table = (Node<K, V>[]) new Node[TABLE_SIZE];
    }

    public void put(K key, V value) {
        int index = Math.abs(key.hashCode()) % TABLE_SIZE;

        if (table[index] == null) {
            table[index] = new Node<>(key, value, null);

        } else {
            Node<K, V> node = table[index];
            // key中第一个节点
            if (node.key.equals(key)) {
                node.value = value;
                return;
            }

            while (node.next != null) {
                // key对应的值已存在，直接替换
                if (node.next.key.equals(key)) {
                    node.next.value = value;
                    size++;

                    return;
                }
                node = node.next;
            }

            // key对应值不存在，插入
            node.next = new Node<>(key, value, null);
        }
    }

    public V get(K key) {
        int index = Math.abs(key.hashCode()) % TABLE_SIZE;

        Node<K, V> node = table[index];
        while (node != null) {
            if (node.key.equals(key)) {
                return node.value;
            }
            node = node.next;
        }

        return null;
    }

    public void delete(K key) {
        int index = Math.abs(key.hashCode()) % TABLE_SIZE;

        Node<K, V> node = table[index];
        while (node != null) {
            if (node.key.equals(key)) {
                table[index] = node.next;

                node.value = null;
                node.next = null;

                size--;

                return;
            }

            node = node.next;
        }
    }

    public List<V> allValues() {
        List<V> result = new ArrayList<>();

        for (Node<K, V> node : table) {
            while (node != null) {
                result.add(node.value);
                node = node.next;
            }
        }

        return result;
    }

    public List<K> allKeys() {
        List<K> result = new ArrayList<>();

        for (Node<K, V> node : table) {
            while (node != null && !result.contains(node.key)) {
                result.add(node.key);
                node = node.next;
            }
        }

        return result;
    }

    private static class Node<K, V> implements Serializable {
        private final K key;

        private V value;
        private Node<K, V> next;

        public Node(K key, V value, Node<K, V> next) {
            this.key = key;
            this.value = value;
            this.next = next;
        }
    }
}
