package net.lensfrex.disillusion.data;

import java.util.Arrays;
import java.util.Objects;

public class LinkedList<T> implements List<T> {
    private int size = 0;

    private Node<T> first;
    private Node<T> last;

    public LinkedList() {
    }

    @Override
    public T get(int index) {
        return node(index).data;
    }

    private Node<T> node(int index) {
        if (index < size / 2) {
            return getNodeForward(index);
        } else {
            return getNodeBackward(index);
        }
    }

    private Node<T> getNodeForward(int index) {
        Node<T> node = first;
        for (int i = 0; i < index; i++) {
            node = node.next;
        }

        return node;
    }

    private Node<T> getNodeBackward(int index) {
        Node<T> node = last;
        for (int i = size - 1; i > index; i--) {
            node = node.prev;
        }

        return node;
    }

    @Override
    public int size() {
        return this.size;
    }

    @Override
    public boolean isEmpty() {
        return this.size == 0;
    }

    @Override
    public void add(T value) {
        Node<T> newNode = new Node<>(this.last, value, null);
        // 空表
        if (this.first == null) {
            this.first = newNode;
        } else {
            this.last.next = newNode;
        }

        this.last = newNode;
        size++;
    }

    @Override
    public void add(T value, int index) {
        checkPositionIndex(index);
        if (index == size) {
            add(value);
        }

        Node<T> nativeNode = node(index);
        Node<T> prev = nativeNode.prev;

        Node<T> newNode = new Node<>(prev, value, nativeNode);
        nativeNode.prev = newNode;

        // 表头位置
        if (prev == null) {
            first = newNode;
        } else {
            prev.next = newNode;
        }

        size++;
    }

    @Override
    public void set(int index, T newValue) {
        Node<T> node = node(index);
        node.data = newValue;
    }

    @Override
    public void remove(int index) {
        checkElementIndex(index);

        unlink(node(index));
        size--;
    }

    @Override
    public void remove(Object value) {
        Node<T> node = first;
        while (node.next != null) {
            if (Objects.equals(value, node.data)) {
                unlink(node);
                return;
            }

            node = node.next;
        }
    }

    private void unlink(Node<T> node) {
        Node<T> prev = node.prev;
        Node<T> next = node.next;

        // 表头位置
        if (prev == null) {
            this.first = next;
        } else {
            prev.next = next;
            node.next = null;
        }

        // 表尾位置
        if (next == null) {
            this.last = prev;
        } else {
            next.prev = prev;
            node.prev = null;
        }

        node.data = null;
    }

    @Override
    public int indexOf(Object value) {
        Node<T> node = first;
        if (node == null) {
            return -1;
        }

        for (int i = 0; node.next != null; i++) {
            if (Objects.equals(value, node.data)) {
                return i;
            }

            node = node.next;
        }

        return -1;
    }

    @Override
    @SuppressWarnings("unchecked")
    public T[] toArray() {
        Object[] data = new Object[size];

        Node<T> node = first;
        for (int i = 0; i < size; i++) {
            data[i] = node;
            node = node.next;
        }

        return (T[]) Arrays.copyOf(data, data.length);
    }

    @Override
    public T[] toArray(T[] array) {
        return null;
    }

    @Override
    public boolean contains(Object obj) {
        Node<T> node = first;
        for (int i = 0; i < size; i++) {
            if (obj.equals(node.data)){
                return true;
            }

            node = node.next;
        }

        return false;
    }

    private void checkPositionIndex(int index) {
        if (index > size) {
            throw new IndexOutOfBoundsException(String.format("Index %s out of %s", index, size));
        }
    }

    private void checkElementIndex(int index) {
        if (index >= size) {
            throw new IndexOutOfBoundsException(String.format("Index %s out of %s", index, size));
        }
    }

    private static class Node<T> {
        Node<T> prev;
        T data;
        Node<T> next;

        public Node(Node<T> prev, T data, Node<T> next) {
            this.prev = prev;
            this.data = data;
            this.next = next;
        }
    }
}
