package net.lensfrex.disillusion.dao.database;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 虚拟数据库底层
 */
class DatabaseOperator {
    private static String baseDir;

    private static boolean initial = false;

    private static final Map<String, Path> paths = new ConcurrentHashMap<>();

    private static final Map<String, Object> tableLocks = new ConcurrentHashMap<>();

    public static <T> void write(String table, T data) throws IOException {
        ObjectOutputStream outputStream = new ObjectOutputStream(Files.newOutputStream(Path.of(baseDir, table)));
        outputStream.writeObject(data);
        outputStream.flush();
        outputStream.close();

        tableLocks.remove(table);
    }

    public static <T> T read(String table, Class<T> type) throws IOException {
        if (!tableLocks.containsKey(table)) {
            tableLocks.put(table, new Object());
        }

        Path path = getPath(table);

        if (!Files.exists(path)) {
            Files.createFile(path);
            return null;
        }

        try {
            ObjectInputStream inputStream = new ObjectInputStream(Files.newInputStream(getPath(table)));
            @SuppressWarnings("unchecked") T result = (T) inputStream.readObject();
            inputStream.close();

            tableLocks.remove(table);

            return result;
        } catch (EOFException e) {
            return null;
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException("IOException");
        }
    }

    private static Path getPath(String table) {
        if (!paths.containsKey(table)) {
            paths.put(table, Path.of(baseDir, table));
        }

        return paths.get(table);
    }

    protected static void init(String baseDir) {
        if (initial) {
            return;
        }

        DatabaseOperator.baseDir = baseDir;
        Paths.get(baseDir).toFile().mkdirs();
        initial = true;
    }
}
